FUNCTION DIALOG_OUTPUTFILE, DIALOG_PARENT = DIALOG_PARENT, $
  GET_PATH = GET_PATH, _EXTRA = EXTRA

;+
; Purpose:
;    DIALOG_OUTPUTFILE allows the user to interactively pick an output file
;    using the platform's own native graphical file-selection dialog.
;    Features:
;    (1) If the user selects an existing file, the user is asked if the file
;        should be overwritten.
;    (2) The selected output file is checked to make sure it can be written.
;        If not, the user is warned and asked to select another file.
;
; Input:
;    None
;
; Output:
;    DIALOG_OUTPUTFILE    String containing the full path name of the
;                         selected output file.
;
; Optional Keywords:
;    DIALOG_PARENT        Set this keyword to the widget ID of a widget
;                         over which any message dialogs should be positioned.
;
;    Also accepts any keywords recognized by DIALOG_PICKFILE.
;
; Example:
; openw, lun, 'zztestit.dat', /get_lun & free_lun, lun
; file = dialog_outputfile( filter = 'zz*.dat' )
; help, file
;
; Author:
;    Liam.Gumley@ssec.wisc.edu
;-

rcs_id = "$Id: dialog_outputfile.pro,v 1.4 1999/06/17 21:43:06 gumley Exp $"

;- Check keywords

if n_elements( dialog_parent ) ne 1 then dialog_parent = 0

;- Call dialog_pickfile to select a file name (return if no file selected)
;  (note that get_path is included because extra keyword values are not returned)

select_file:
filename = dialog_pickfile( get_path = get_path, _extra = extra )
if filename eq '' then return, filename

;- Test whether file exists

if ( findfile( filename ) )[0] eq '' then begin

  ;- File does not exist, so check that it can be opened for writing

  if not filecheck( filename, /write ) then begin
    text = filename + ' could not be created. Please use a different filename.'
    result = dialog_message( text, /error, dialog_parent = dialog_parent )
    goto, select_file
  endif

endif else begin

  ;- File exists, so ask user if it should be overwritten

  text = filename + ' already exists. Overwrite it?'
  result = dialog_message( text, /question, dialog_parent = dialog_parent )
  if result eq 'No' then goto, select_file

  ;- Check that it can be opened for writing

  if not filecheck( filename, /write ) then begin
    text = filename + ' could not be opened. Please use a different filename.'
    result = dialog_message( text, /error, dialog_parent = dialog_parent )
    goto, select_file
  endif

endelse

;- Return result to caller

return, filename

END
