FUNCTION FILE_HISTORY, TITLE, FILES, $
  SET=SET, MAXFILES=MAXFILES, MESSAGE=MESSAGE

;+
; PURPOSE:
;    To set or get a file history list (the most recently opened files).
;
; USAGE:
;    RESULT = FILE_HISTORY(TITLE, FILES)
;
; INPUT:
;    TITLE     Full path and name of file history list.
;    FILES     Array of names of files retrieved from the file,
;              or to be added to the file
;              (non-unique filenames are removed from the history list).
;
; OUTPUT:
;    RESULT    Success/Fail flag
;              1 => Success
;              0 => Fail
;
; KEYWORDS:
;    SET       If set, add the files in FILES to the history list
;              (default is to get the history list and store it in FILES)
;    MAXFILES  The maximum number of entries permitted in the history list
;              (default is 5)
;    MESSAGE   If set to a named variable, and the fail flag is returned,
;              contains a string describing the reaons for the failure.
;
; EXAMPLE:
; result = file_history('.sharp_history.txt', 'file01.dat', /set)
; result = file_history('.sharp_history.txt', files)
; print, files
; result = file_history('.sharp_history.txt', 'file02.dat', /set)
; result = file_history('.sharp_history.txt', files)
; print, files
;
; REVISED:
; Liam.Gumley@ssec.wisc.edu
; $Id: file_history.pro,v 1.5 1999/09/01 15:09:04 gumley Exp $
;-

rcs_id = '$Id: file_history.pro,v 1.5 1999/09/01 15:09:04 gumley Exp $

;- Check arguments

if n_params() ne 2 then message, 'Usage: RESULT = FILE_HISTORY(TITLE, FILES)'
if n_elements(title) eq 0 then message, 'Argument TITLE is undefined'

;- Check keywords

if not keyword_set(set) then set = 0
if n_elements(maxfiles) eq 0 then maxfiles = 5

;- Check input values

if set and n_elements(files) eq 0 then message, 'Argument FILES is undefined'

;- Set success and fail flag values

success = 1
fail = 0

;- If we are in demo mode, then return fail code

if check_demo_mode() then return, fail

;- Get directory for preferences file

case !version.os_family of
  'MacOS'   : root = !dir
  'Windows' : root = !dir
  'vms'     : root = 'SYS$LOGIN'
  'unix'    : root = '~'
endcase

;- Append the directory delimiter for this OS

root = dir_path(root)

;- Construct file path

filepath = root + strcompress(title, /remove_all)

;-------------------------------------------------------------------------------
;- GET THE CONTENTS OF AN EXISTING FILE
;-------------------------------------------------------------------------------

if not set then begin

  ;- If file is not readable, exit
  
  if not filecheck(filepath) then begin
    message = 'History file ' + filepath + ' was not found'
    return, fail
  endif
  
  ;- Read the file
  
  files = strarr(1000)
  string = ' '
  count = 0
  openr, lun, filepath, /get_lun
  while not eof(lun) do begin
    readf, lun, string, format='(a)'
    if strlen(string) gt 0 then begin
      files[count] = string
      count = count + 1
    endif
  endwhile
  free_lun, lun
  files = files[0:(count < maxfiles)-1]

  ;- Remove any non-unique files

  files = unique(files)
  count = n_elements(files)

  ;- Make sure the files are still readable
  
  exist = lonarr(count)
  for i = 0, count - 1 do begin
    exist[i] = filecheck(files[i])
  endfor
  index = where(exist, count)
  if count gt 0 then begin
    files = files[index]
  endif else begin
    files = ''
  endelse
    
  ;- Return the list of files
  
  if count gt 0 then begin
    return, success
  endif else begin
    message = 'History file ' + filepath + ' is empty'
    return, fail
  endelse
    
endif

;-------------------------------------------------------------------------------
;- CREATE A NEW FILE OR UPDATE AN EXISTING FILE
;-------------------------------------------------------------------------------

;- If file is not writable, exit

if not filecheck(filepath, /write) then $
  message = 'History file ' + filepath + ' cannot be written'

;- Read an existing file

if filecheck(filepath) then begin

  ;- Read the file

  old_files = strarr(1000)
  string = ' '
  count = 0
  openr, lun, filepath, /get_lun
  while not eof(lun) do begin
    readf, lun, string, format='(a)'
    if strlen(string) gt 0 then begin
      old_files[count] = string
      count = count + 1
    endif
  endwhile
  free_lun, lun
  old_files = old_files[0:(count < maxfiles)-1]

  ;- If no filenames were found, return fail code

  if count eq 0 then begin
    message = 'History file ' + filepath + ' is empty'
    return, fail
  endif

endif

;- Add the new file to the old file list

if n_elements(old_files) eq 0 then begin
 old_files = files
endif else begin
  old_files = [files, old_files]
endelse
count = n_elements(old_files)
old_files = old_files[0:(count < maxfiles)-1]

;- Remove any non-unique files

old_files = unique(old_files)
count = n_elements(old_files)

;- Write the file list

openw, lun, filepath, /get_lun
i = 0
while i lt count do begin
  printf, lun, old_files[i], format='(a)'
  i = i + 1
endwhile
free_lun, lun

;- Return success flag

return, success

END
