FUNCTION MAS_FILE_TYPE, FILE, NSCANS = NSCANS

rcs_id = "$Id: mas_file_type.pro,v 1.4 1998/08/07 20:29:25 gumley Exp $"

;+
; Purpose:
;    Identify the format of a MAS file. Recognized formats are
;    (1) Exabyte format as recorded in-flight (57344 bytes per block),
;    (2) UW-Intermediate format (74200 bytes per block),
;    (3) HDF Level 1B format,
;    (4) UW-Intermediate quicklook format.
;    An error message will be printed and execution will halt if the file
;    cannot be opened.
;
; Usage:
;    TYPE = MAS_FILE_TYPE( FILE )
;
; Input:
;    FILE             Name of file
;
; Optional Keywords:
;    NSCANS           Returns number of scans in MAS file
;
; Output:
;    MAS_FILE_TYPE    Integer variable identifying file format
;                     0 => File is not a recognized MAS format
;                     1 => Exabyte
;                     2 => UW-Intermediate
;                     3 => HDF Level 1B
;                     4 => UW Intermediate quicklook format
;
; Revised:
;    30-SEP-1996 Liam Gumley, CIMSS/SSEC
;                Created
;    27-NOV-1996 Liam Gumley, CIMSS/SSEC
;                Fixed problem with decoding scan number from UW format
;    07-APR-1997 Liam Gumley, CIMSS/SSEC
;                Added identification of UW Intermediate quicklook format.
;                Added test for byte-swapped UW Intermediate data types.
;    22-APR-1997 Liam Gumley, CIMSS/SSEC
;                Added NSCANS keyword to return number of scans.
;    23-APR-1997 Liam Gumley, CIMSS/SSEC
;                Made test for UW Intermediate quicklook format more robust.
;    24-APR-1997 Liam Gumley, CIMSS/SSEC
;                Made test for MAS HDF faster and more robust.
;    26-SEP-1997 Liam Gumley, CIMSS/SSEC
;                Added CATCH error handler so this program will not crash,
;                which allows the function to handle non-MAS HDF files.
;-

;- Set return values

result = 0
nscans = 0

;- Set error handler

catch, error_status
if error_status ne 0 then return, result

;- Check if file is MAS HDF

if hdf_ishdf( file ) then begin

  sd_id = hdf_sd_start( file )
  index = hdf_sd_nametoindex( sd_id, 'CalibratedData' )

  if index eq -1 then begin

    hdf_sd_end, sd_id

  endif else begin

    sds_id = hdf_sd_select( sd_id, index )
    hdf_sd_getinfo, sds_id, dims = dims
    npixels = dims( 0 )
    nscans = dims( 2 )
    hdf_sd_endaccess, sds_id
    hdf_sd_end, sd_id
    result = 3
    return, result
    
  endelse

endif

;- Get file size information

openr, lun, file, /get_lun
info = fstat( lun )
free_lun, lun

;- Check if file is raw Exabyte format (57344 bytes per scan)

if info.size ge 57344 then begin

  openr, lun, file, /get_lun
  data = bytarr( 1126 )
  readu, lun, data
  free_lun, lun
  if data( 0 ) eq 'A6'XB and data( 1 ) eq 'AF'XB and $
    data( 0 + 1124 ) eq 'A6'XB and data( 1 + 1124 ) eq 'AF'XB then begin
    result = 1
    nscans = long( info.size ) / 57344L
    return, result
  endif

endif

;- Check if file is UW intermediate format (74200 bytes per scan)

if info.size ge 1492 then begin

  openr, lun, file, /get_lun
  data = lonarr( 373 )
  readu, lun, data
  free_lun, lun
  if data( 1 ) lt 0L or data( 1 ) gt 270000L then begin
    data = swap_endian( temporary( data ) )
  endif
  scan1 = data( 1 )
  scan2 = data( 372 )
  if ( scan2 eq scan1 or scan2 eq ( scan1 + 1 ) or scan2 eq ( scan1 + 2 ) ) $
    and scan1 ge 1000 and scan1 le 270000L then begin
    result = 2
    nscans = long( info.size ) / 74200L
    return, result
  endif

endif

;- Check if file is UW intermediate quicklook format (408 bytes per scan)

if info.size ge 416 then begin

  openr, lun, file, /get_lun
  data = lonarr( 2 )
  readu, lun, data
  if data( 1 ) lt 0L or data( 1 ) gt 270000L then begin
    data = swap_endian( temporary( data ) )
  endif
  scan1 = data( 1 )
  point_lun, lun, 408
  readu, lun, data
  if data( 1 ) lt 0L or data( 1 ) gt 270000L then begin
    data = swap_endian( temporary( data ) )
  endif
  scan2 = data( 1 )
  free_lun, lun
  if ( scan2 eq ( scan1 + 4 ) or scan2 eq ( scan1 + 8 ) ) $
    and scan1 ge 1000 and scan1 le 270000L then begin
    result = 4
    nscans = long( info.size ) / 408L
    return, result
  endif

endif

catch, /cancel
return, result

END
