PRO MAS_FLAT_FILE, FILE, BAND, $
  HANDLE=HANDLE, FLIP=FLIP, SCENE=SCENE, TIME=TIME, GEODATA=GEODATA

;+
; PURPOSE:
;    To create a single band binary image file from a MAS HDF file.
;
; USAGE:
;    MAS_FLAT_FILE, FILE, BAND
;
; INPUT:
;    FILE      Name of MAS HDF file
;    BAND      Band number to extract
;
; OPTIONAL KEYWORDS
;    HANDLE    If set to a string variable, the handle used for output files
;              (default is 'mas')
;    FLIP      If set, flips the byte order of the output files
;              (default is do not flip byte order)
;    SCENE     If set to a named variable, specifies the along-track scene
;              start and end record numbers
;              (default is [0,715])
;    TIME      If set to a named variable, specifies the time code at
;              the center of the scene to be extracted (HHMMSS)
;              (over-rides SCENE, and a square 716x716 scene centered
;               at the requested time is read)
;    GEODATA   If set, writes the geolocation data (latitude, longitude,
;              solar zenith, solar azimuth, sensor zenith, sensor azimuth)
;              for the scene to separate files, each in 716x716 32-bit
;              floating point format (see mas_read.pro for geolocation
;              data definitions and units).
;
; OUTPUT:
;    Creates 3 output files in 716x716 32-bit floating point format:
;        mas.c??    Image data for the requested scene where
;                   ?? is MAS band number,
;                   Units are reflectance (no units) for bands 1-25,
;                   Units are brightness temperature (K) for bands 26-50.
;        mas.lat    Latitude data for the scene (deg).
;        mas.lon    Longitude data for the scene (deg, -180W to +180E).
;
;    Creates 1 output file in ASCII format:
;        mas.tim    Contains date/time/name information for the scene,
;                   and scanline/time information for each line.
;
; AUTHOR:
;    Liam.Gumley@ssec.wisc.edu
;    $Id: mas_flat_file.pro,v 1.5 1999/06/18 16:18:40 gumley Exp $
;
; RESTRICTIONS:
;    Requires the following support routines which are available at
;    ftp://origin.ssec.wisc.edu/pub/MAS/idl:
;    bright_m.pro
;    mas_file_type.pro
;    mas_read.pro
;
; EXAMPLE:
;; When running SHARP (http://cimss.ssec.wisc.edu/~gumley/sharp/sharp.html)
;; you will notice that the time code (HHMMSS) at the center of the scene
;; is displayed as part of the title. For an example, click on the leftmost
;; screenshot at the top left. The time code can then be used in the call to
;; MAS_FLAT_FILE to specify the center of the scene to be extracted, e.g.
;
;file = 'MAS_950607_01.hdf'
;band = 2
;mas_flat_file, file, band, time=201255, handle='950607_201255'
;-

rcs_id = "$Id: mas_flat_file.pro,v 1.5 1999/06/18 16:18:40 gumley Exp $"

;- Check arguments

if n_elements( file ) eq 0 then message, 'Argument FILE is missing'
if n_elements( band ) eq 0 then message, 'Argument BAND is missing'

;- Check keywords

if n_elements( handle ) eq 0 then handle = 'mas'
if not keyword_set( flip ) then flip = 0
if n_elements( scene ) eq 0 then scene = [ 0, 715 ]
if not keyword_set( geodata ) then geodata = 0

;- Check if scene center time was specified

if n_elements( time ) eq 1 then begin

  ;- Get time data (HHMMSS) for entire flight

  sd_id = hdf_sd_start( file )
  index = hdf_sd_nametoindex( sd_id, 'GpsTime' )
  if index lt 0 then index = hdf_sd_nametoindex( sd_id, 'GreenwichMeanTime' )
  sds_id = hdf_sd_select( sd_id, index )
  hdf_sd_getdata, sds_id, timecode
  hdf_sd_endaccess, sds_id
  hdf_sd_end, sd_id
  timecode = temporary( timecode ) / 100L

  ;- Find best time match
  
  minv = min( abs( time - timecode ), loc )
  scan1 = ( ( loc - 358 ) > 0 ) < ( n_elements( timecode ) - 1 - 715 )
  scan2 = scan1 + 715
  scene = [ scan1, scan2 ]
  
endif
  
;- Read data from MAS HDF file with or without geolocation data,
;- and byte flip if requested

if geodata then begin

  mas_read, file, band, image, date, start_time, end_time, $
    scene = scene, time = timecode, scanline = scanline, track = track, $
    lat = lat, lon = lon, $
    solzen = solzen, solazm = solazm, $
    senzen = senzen, senazm = senazm
  if flip then begin
    image  = swap_endian( temporary( image ) )
    lat    = swap_endian( temporary( lat ) )
    lon    = swap_endian( temporary( lon ) )
    solzen = swap_endian( temporary( solzen ) )
    solazm = swap_endian( temporary( solazm ) )
    senzen = swap_endian( temporary( senzen ) )
    senazm = swap_endian( temporary( senazm ) )
  endif
  
endif else begin

  mas_read, file, band, image, date, start_time, end_time, $
    scene = scene, time = timecode, scanline = scanline, track = track
  if flip then image = swap_endian( temporary( image ) )
  
endelse

;- Write image data to flat file

outimg = string( handle ) + string( band, format = '( ".c", i2.2 )' )
openw, lun, outimg, /get_lun & writeu, lun, image & free_lun, lun

;- Write scanline, time, and date information to ASCII file

outtim = string( handle ) + '.time'
openw, lun, outtim, /get_lun
printf, lun, '# File : ' + file
printf, lun, '# Band : ' + strcompress( band )
printf, lun, '# Date : ' + date
printf, lun, '# Track: ' + strcompress( track )
printf, lun, '# Record   Scan   HHMMSS'
for i = 0, n_elements( scanline ) - 1 do begin
  printf, lun, i, scanline( i ), timecode( i ), $
    format = '( 2x, i6, 2x, i6, 2x, i6.6 )'
endfor
free_lun, lun

;- Write geolocation data if it was requested

if geodata then begin
  outlat    = string( handle ) + '.lat'
  outlon    = string( handle ) + '.lon'
  outsolzen = string( handle ) + '.solzen'
  outsolazm = string( handle ) + '.solazm'
  outsenzen = string( handle ) + '.senzen'
  outsenazm = string( handle ) + '.senazm'
  openw, lun, outlat,    /get_lun & writeu, lun, lat    & free_lun, lun
  openw, lun, outlon,    /get_lun & writeu, lun, lon    & free_lun, lun
  openw, lun, outsolzen, /get_lun & writeu, lun, solzen & free_lun, lun
  openw, lun, outsolazm, /get_lun & writeu, lun, solazm & free_lun, lun
  openw, lun, outsenzen, /get_lun & writeu, lun, senzen & free_lun, lun
  openw, lun, outsenazm, /get_lun & writeu, lun, senazm & free_lun, lun
endif

;- Report what happened

dims = size( image, /dimensions )
print, " "
print, 'File : ' + file
print, 'Band : ' + strcompress( band )
print, 'Date : ' + date
print, 'Track: ' + strcompress( track )
print, dims, outimg, $
  format='( "Wrote ", i6, " pixels by ", i6, " lines to ", a, " (REAL*4)" )'
print, " "

END
