PRO MAS_HDF_READ, FILE, RECORD, NRECORDS, IMAGE, DATE, TIME, SCANLINE, $
  LAT, LON, SOLZEN, SOLAZM, SENZEN, SENAZM, UNITS = UNITS

rcs_id = "$Id: mas_hdf_read.pro,v 1.1 1998/10/22 21:37:55 gumley Exp $"

;+
; NAME:
;    MAS_HDF_READ
;
; PURPOSE:
;    Read all bands of MODIS Airborne Simulator (MAS) image data from a
;    Level 1B HDF file.
;
; CATEGORY:
;    MAS utilities.
;
; CALLING SEQUENCE:
;    MAS_HDF_READ, FILE, RECORD, NRECORDS, IMAGE, DATE, TIME, SCANLINE, $
;      LAT, LON, SOLZEN, SOLAZM, SENZEN, SENAZM
;
; INPUTS:
;    FILE        Name of the MAS Level 1B HDF file, or UW intermediate format file
;    RECORD      Start record number (zero based)
;    NRECORDS    Number of records to read
;    
; OPTIONAL INPUTS:
;    None
;
; KEYWORD PARAMETERS:
;    UNITS       If set to a named variable, specifies the units to be returned
;                (default is 0)
;                UNITS=0 => Reflectance (no units) for bands 1-25
;                           Brightness Temperature (K) for bands 26-50
;                UNITS=1 => Radiance for bands 1-50 (Watts per sq. meter per steradian per micron)
;
; OUTPUTS:
;    IMAGE       Named varibale in which an array of image data is returned. 
;                Dimensions are [716,NBANDS,NRECORDS]. NBANDS is determined by the number
;                of bands present in the input file (12 or 50).
;                Units are determined by the UNITS keyword
;    DATE        Named variable in which a vector of dates (YYYYMMDD) is returned
;    TIME        Named variable in which a vector of times (HHMMSS) is returned
;    LAT         Named variable in which a 2D array of latitudes (deg)
;                is returned
;    LON         Named variable in which a 2D array of longitudes (deg, -180W to +180E)
;                is returned
;    SOLZEN      Named variable in which a 2D array of solar zenith angles (deg)
;                is returned (zenith angle of a vector from the pixel to the Sun)
;    SOLAZM      Named variable in which a 2D array of solar azimuth angles (deg)
;                is returned (azimuth angle clockwise from North of a vector from the pixel to the Sun)
;    SENZEN      Named variable in which a 2D array of sensor zenith angles (deg)
;                is returned (zenith angle of a vector from the sensor to the pixel)
;    SENAZM      Named variable in which a 2D array of sensor azimuth angles (deg)
;                is returned (azimuth angle clockwise from North of a vector from the pixel to the sensor)
;
; OPTIONAL OUTPUTS:
;    None
;
; COMMON BLOCKS:
;    None
;
; SIDE EFFECTS:
;    None
;
; RESTRICTIONS:
;    None
;
; EXAMPLE:
;-

;- Return to caller if error occurs

on_error, 2

;- Check keywords

if n_elements( units ) ne 1 then units = 0

;- Open file

sd_id = hdf_sd_start( file )

;- Get time (HHMMSS)

index = hdf_sd_nametoindex( sd_id, 'GpsTime' )
if index lt 0 then index = hdf_sd_nametoindex( sd_id, 'GreenwichMeanTime' )
sds_id = hdf_sd_select( sd_id, index )
hdf_sd_getdata, sds_id, time, start = [ record ], count = [ nrecords ]
hdf_sd_endaccess, sds_id
time = temporary( time ) / 100L

;- Get date (YYYYMMDD)

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'YearMonthDay' ) )
hdf_sd_getdata, sds_id, date, start = [ record ], count = [ nrecords ]
hdf_sd_endaccess, sds_id

;- Get scanline counter

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'ScanLineCounter' ) )
hdf_sd_getdata, sds_id, scanline, start = [ record ], count = [ nrecords ]
hdf_sd_endaccess, sds_id

;- Get latitude and longitude

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'PixelLatitude' ) )
hdf_sd_getinfo, sds_id, dims = dims
hdf_sd_getdata, sds_id, lat, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'PixelLongitude' ) )
hdf_sd_getdata, sds_id, lon, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

;- Get solar zenith and azimuth

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarZenithAngle' ) )
hdf_sd_getdata, sds_id, solzen, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarAzimuthAngle' ) )
hdf_sd_getdata, sds_id, solazm, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

;- Get sensor zenith and azimuth

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SensorZenithAngle' ) )
hdf_sd_getdata, sds_id, senzen, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SensorAzimuthAngle' ) )
hdf_sd_getdata, sds_id, senazm, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id

;- Get solar spectral irradiances

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarSpectralIrradiance' ) )
hdf_sd_getdata, sds_id, irradiance
hdf_sd_endaccess, sds_id

;- Get center wavelengths

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'Central100%ResponseWavelength' ) )
hdf_sd_getdata, sds_id, center
hdf_sd_endaccess, sds_id
index = hdf_sd_nametoindex( sd_id, 'EffectiveCentralWavelength_IR_Bands' )
if index gt 0 then begin
  sds_id = hdf_sd_select( sd_id, index )
  hdf_sd_getdata, sds_id, tc_center
  hdf_sd_endaccess, sds_id
  center( 25 : 49 ) = temporary( tc_center( 25 : 49 ) )
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionSlope' ) )
  hdf_sd_getdata, sds_id, tc_slope
  hdf_sd_endaccess, sds_id
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionIntercept' ) )
  hdf_sd_getdata, sds_id, tc_intercept
  hdf_sd_endaccess, sds_id
endif
nbands = n_elements( center )

;- Get image data for all bands

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
start = [ 0, 0, record ]
count = [ 716, nbands, nrecords ]
hdf_sd_getdata, sds_id, image, start = start, count = count
hdf_sd_attrinfo, sds_id, hdf_sd_attrfind( sds_id, 'scale_factor' ), data = scale
hdf_sd_endaccess, sds_id

;- Close file

hdf_sd_end, sd_id

;- Apply scale factors to all bands

image = float( temporary( image ) )
for band = 1, nbands do begin
  image( *, band - 1, * ) = image( *, band - 1, * ) * scale( band - 1 )
endfor

;- Make sure there are no negative values in the image

image = temporary( image ) > 1.0e-6

;- Resample geolocation arrays if necessary

sz = size( lat )
if sz( 1 ) lt 716 then begin
  lat = congrid( temporary( lat ), 716, nrecords, /interp )
  lon = congrid( temporary( lon ), 716, nrecords, /interp )
  solzen = congrid( temporary( solzen ), 716, nrecords, /interp )
  solazm = congrid( temporary( solazm ), 716, nrecords, /interp )
  senzen = congrid( temporary( senzen ), 716, nrecords, /interp )
  senazm = congrid( temporary( senazm ), 716, nrecords, /interp )
endif

;- Rotate 2D arrays into correct orientation

lat    = rotate( rotate( temporary( lat    ), 7 ), 2 )
lon    = rotate( rotate( temporary( lon    ), 7 ), 2 )
solzen = rotate( rotate( temporary( solzen ), 7 ), 2 )
solazm = rotate( rotate( temporary( solazm ), 7 ), 2 )
senzen = rotate( rotate( temporary( senzen ), 7 ), 2 )
senazm = rotate( rotate( temporary( senazm ), 7 ), 2 )
for band = 1, nbands do begin
  image( *, band - 1, * ) = rotate( rotate( reform( image( *, band - 1, * ) ), 7 ), 2 )
endfor

;- Convert image to reflectance/temperature unless UNITS keyword is set

if units eq 0 then begin

  ;- Loop over all bands
  
  for band = 1, nbands do begin

    if center( band - 1 ) lt 2.5 then begin

      ;- Convert visible/near-IR bands from radiance to reflectance
      
      image( *, band - 1, * ) = image( *, band - 1, * ) * $
        ( !pi / ( irradiance( band - 1 ) * cos( !dtor * solzen ) ) )

    endif else begin

      ;- Convert IR bands from radiance to brightness temperature

      image( *, band - 1, * ) = bright_m( center( band - 1 ), image( *, band - 1, * ) )
      if n_elements( tc_slope ) eq n_elements( center ) then $
	  image( *, band - 1, * ) = ( image( *, band - 1, * ) - tc_intercept( band - 1 ) ) / $
          tc_slope( band - 1 )

    endelse

  endfor

endif

END
