PRO MAS_READ, FILE, BAND, IMAGE, DATE, TIME1, TIME2, $
  SCENE = SCENE, UNITS = UNITS, CENTER = CENTER, SCANLINE = SCANLINE, $
  NSCANS = NSCANS, TIME = TIME, LAT = LAT, LON = LON, SOLZEN = SOLZEN, $
  SOLAZM = SOLAZM, SENZEN = SENZEN, SENAZM = SENAZM, $
  EXPNAME = EXPNAME, TRACK = TRACK, HEADING = HEADING

rcs_id = "$Id: mas_read.pro,v 1.15 1999/07/19 15:32:35 gumley Exp $"

;+
; NAME:
;    MAS_READ
;
; PURPOSE:
;    Read a single band of MODIS Airborne Simulator (MAS) image data from a
;    Level 1B HDF file.
;
; CATEGORY:
;    MAS utilities.
;
; CALLING SEQUENCE:
;    MAS_READ, FILE, BAND, IMAGE, DATE, TIME1, TIME2
;
; INPUTS:
;    FILE        Name of the MAS Level 1B HDF file, or UW intermediate format file
;    BAND        MAS band number to extract (1-50)
;    
; OPTIONAL INPUTS:
;    None
;	
; KEYWORD PARAMETERS:
;    SCENE       If set to a named variable, specifies the along-track scene start and end
;                (default is [0,715])
;                Note that 716 pixels across-track are always extracted
;    UNITS       If set to a named variable, specifies the units to be returned
;                (default is 0)
;                UNITS=0 => Reflectance (no units) for bands 1-25
;                           Brightness Temperature (K) for bands 26-50
;                UNITS=1 => Radiance for bands 1-50 (Watts per sq. meter per steradian per micron)
;                UNITS=2 => Digital Counts for bands 1-50
;    CENTER      Named variable in which a vector of central wavelengths (microns)
;                is returned for bands 1-50
;    SCANLINE    Named variable in which a vector of scanline counter values
;                is returned for the scene
;    NSCANS      Named variable in which the total number of scanlines
;                is returned for the entire file
;    TIME        Named variable in which a vector of times (HHMMSS)
;                is returned for the scene
;    LAT         Named variable in which a 2D array of latitudes (deg)
;                is returned for the scene
;    LON         Named variable in which a 2D array of longitudes (deg, -180W to +180E)
;                is returned for the scene
;    SOLZEN      Named variable in which a 2D array of solar zenith angles (deg)
;                is returned for the scene
;                (zenith angle of a vector from the pixel to the Sun)
;    SOLAZM      Named variable in which a 2D array of solar azimuth angles (deg)
;                is returned for the scene
;                (azimuth angle clockwise from North of a vector from the pixel to the Sun)
;    SENZEN      Named variable in which a 2D array of sensor zenith angles (deg)
;                is returned for the scene
;                (zenith angle of a vector from the sensor to the pixel)
;    SENAZM      Named variable in which a 2D array of sensor azimuth angles (deg)
;                is returned for the scene
;                (azimuth angle clockwise from North of a vector from the pixel to the sensor)
;    EXPNAME     Named variable in which a string containing the field experiment name
;                is returned for the file
;    TRACK       Named variable in which the flight track number
;                is returned for the file
;    HEADING     Named variable in which a vector of aircraft heading values
;                is returned for the scene
;
; OUTPUTS:
;    IMAGE       Array of image data. Units are determined by the UNITS keyword
;    DATE        Date (UTC) for first MAS scanline in IMAGE (YYYYMMDD)
;    TIME1       Time (UTC) for first MAS scanline in IMAGE (HHMMSS)
;    TIME2       Time (UTC) for last MAS scanline in IMAGE (HHMMSS)
;
; OPTIONAL OUTPUTS:
;    None
;
; COMMON BLOCKS:
;    None
;
; SIDE EFFECTS:
;    None
;
; RESTRICTIONS:
;    None
;
; EXAMPLE:
; MAS_READ, 'MAS_960502_28.hdf', 2, IMAGE, DATE, TIME1, TIME2
; HELP, IMAGE
; PRINT, DATE, " ", TIME1, " ", TIME2
;-

;- Return to caller if error occurs

on_error, 2

;- Check keywords

if n_elements( scene ) ne 2 then scene = [ 0, 715 ]
if n_elements( units ) ne 1 then units = 0

;- Check that file exists

list = findfile( file )
if list( 0 ) eq '' then begin
  help, file
  message, 'Argument FILE was not found'
endif
 
;- Check band argument

if band lt 1 or band gt 50 then begin
  help, band
  message, 'Argument BAND must be in the range [1-50]'
endif

;- Check file type

if mas_file_type( file ) ne 3 then begin
  help, file
  message, 'Argument FILE was not recognized as a valid MAS HDF format'
endif

;- Open file

sd_id = hdf_sd_start( file )

;- Get experiment name

index =  hdf_sd_attrfind( sd_id, 'ExperimentName' )
hdf_sd_attrinfo, sd_id, index, data = expname
expname = strtrim( expname[ 0 ] )

;- Get flight line number

index =  hdf_sd_attrfind( sd_id, 'FlightLineNumber' )
hdf_sd_attrinfo, sd_id, index, data = track
track = track[ 0 ]

;- Get total number of scanlines

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
hdf_sd_getinfo, sds_id, dims = dims
hdf_sd_endaccess, sds_id
nscans = long( dims( 2 ) )
  
;- Get start and end scan numbers for scene

scan1 = ( scene( 0 ) > 0 ) < ( nscans - 2 )
scan2 = ( scene( 1 ) > ( scan1 + 1 ) ) < ( nscans - 1 )
scene = [ scan1, scan2 ]
scan_count = scan2 - scan1 + 1

;- Get times (HHMMSS)

index = hdf_sd_nametoindex( sd_id, 'GpsTime' )
if index lt 0 then index = hdf_sd_nametoindex( sd_id, 'GreenwichMeanTime' )
sds_id = hdf_sd_select( sd_id, index )
hdf_sd_getdata, sds_id, time, start = [ scan1 ], count = [ scan_count ]
hdf_sd_endaccess, sds_id
time = temporary( time ) / 100L
time1 = time( 0 )
time2 = time( scan_count - 1 )

;- Get start date (YYYYMMDD)

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'YearMonthDay' ) )
hdf_sd_getdata, sds_id, date, start = [ scan1 ], count = [ 1 ]
hdf_sd_endaccess, sds_id
date = date( 0 )

;- Convert integers to strings

date = strcompress( date, /remove_all )
time1 = strcompress( time1, /remove_all )
time2 = strcompress( time2, /remove_all )

;- Get solar spectral irradiances

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarSpectralIrradiance' ) )
hdf_sd_getdata, sds_id, irradiance
hdf_sd_endaccess, sds_id

;- Get center wavelengths

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'Central100%ResponseWavelength' ) )
hdf_sd_getdata, sds_id, center
hdf_sd_endaccess, sds_id
index = hdf_sd_nametoindex( sd_id, 'EffectiveCentralWavelength_IR_Bands' )
if index gt 0 then begin
  sds_id = hdf_sd_select( sd_id, index )
  hdf_sd_getdata, sds_id, tc_center
  hdf_sd_endaccess, sds_id
  center( 25 : 49 ) = temporary( tc_center( 25 : 49 ) )
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionSlope' ) )
  hdf_sd_getdata, sds_id, tc_slope
  hdf_sd_endaccess, sds_id
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionIntercept' ) )
  hdf_sd_getdata, sds_id, tc_intercept
  hdf_sd_endaccess, sds_id
endif

;- Get scanline counter array

if arg_present( scanline ) then begin
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'ScanLineCounter' ) )
  hdf_sd_getdata, sds_id, scanline, start = [ scan1 ], count = [ scan_count ]
  hdf_sd_endaccess, sds_id
endif

;- Get heading array

if arg_present( heading ) then begin
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'AircraftHeading' ) )
  hdf_sd_getdata, sds_id, heading, start = [ scan1 ], count = [ scan_count ]
  hdf_sd_endaccess, sds_id
endif

;- Get calibration slope array

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibrationSlope' ) )
hdf_sd_getdata, sds_id, calslope, start = [ band - 1, scan1 ], count = [ 1, scan_count ]
hdf_sd_endaccess, sds_id

;- Get calibration intercept array

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibrationIntercept' ) )
hdf_sd_getdata, sds_id, calintercept, start = [ band - 1, scan1 ], count = [ 1, scan_count ]
hdf_sd_endaccess, sds_id

;- Get latitude and longitude arrays

if arg_present( lat ) or arg_present( lon ) then begin

  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'PixelLatitude' ) )
  hdf_sd_getinfo, sds_id, dims = dims
  hdf_sd_getdata, sds_id, lat, $
    start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
  hdf_sd_endaccess, sds_id
  if dims( 0 ) lt 716 then lat = congrid( temporary( lat ), 716, scan_count, /interp )
  lat = rotate( rotate( temporary( lat ), 7 ), 2 )

  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'PixelLongitude' ) )
  hdf_sd_getdata, sds_id, lon, $
    start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
  hdf_sd_endaccess, sds_id
  if dims( 0 ) lt 716 then lon = congrid( temporary( lon ), 716, scan_count, /interp )
  lon = rotate( rotate( temporary( lon ), 7 ), 2 )

endif

;- Get solar zenith and azimuth arrays

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarZenithAngle' ) )
hdf_sd_getinfo, sds_id, dims = dims
hdf_sd_getdata, sds_id, solzen, $
  start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
hdf_sd_endaccess, sds_id
if dims( 0 ) lt 716 then solzen = congrid( temporary( solzen ), 716, scan_count, /interp )
solzen = rotate( rotate( temporary( solzen ), 7 ), 2 )

if arg_present( solazm ) then begin
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarAzimuthAngle' ) )
  hdf_sd_getinfo, sds_id, dims = dims
  hdf_sd_getdata, sds_id, solazm, $
    start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
  hdf_sd_endaccess, sds_id
  if dims( 0 ) lt 716 then solazm = congrid( temporary( solazm ), 716, scan_count, /interp )
  solazm = rotate( rotate( temporary( solazm ), 7 ), 2 )
endif

;- Get sensor zenith and azimuth arrays

if arg_present( senzen ) or arg_present( senazm ) then begin

  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SensorZenithAngle' ) )
  hdf_sd_getinfo, sds_id, dims = dims
  hdf_sd_getdata, sds_id, senzen, $
    start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
  hdf_sd_endaccess, sds_id
  if dims( 0 ) lt 716 then senzen = congrid( temporary( senzen ), 716, scan_count, /interp )
  senzen = rotate( rotate( temporary( senzen ), 7 ), 2 )

  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SensorAzimuthAngle' ) )
  hdf_sd_getinfo, sds_id, dims = dims
  hdf_sd_getdata, sds_id, senazm, $
    start = [ 0, scan1 ], count = [ dims( 0 ), scan_count ]
  hdf_sd_endaccess, sds_id
  if dims( 0 ) lt 716 then senazm = congrid( temporary( senazm ), 716, scan_count, /interp )
  senazm = rotate( rotate( temporary( senazm ), 7 ), 2 )

endif

;- Read image and rotate into correct orientation (aircraft flying up the display)
;- Note that a loop is required to work around a problem in IDL 5.2 under Windows

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
start = [ 0, band - 1, scan1 ]
count = [ 716, 1, scan_count ]
data = intarr( count[0], count[2] )
for i = 0, count[2] - 1 do begin
  hdf_sd_getdata, sds_id, record, $
    start = [start[0], start[1], start[2] + i], $
    count = [count[0], count[1], 1]
  data[*, i] = reform(record)
endfor
hdf_sd_attrinfo, sds_id, hdf_sd_attrfind( sds_id, 'scale_factor' ), data = scale
hdf_sd_endaccess, sds_id
image = temporary( data ) * scale( band - 1 )
image = rotate( rotate( temporary( image ), 7 ), 2 )

;- Make sure there are no negative values in the image

image = temporary( image ) > 1.0e-6

;- Convert image to appropriate units

;- Default case (units = 0) is to
;- convert radiance to reflectance (center wavelength < 2.5 microns) or
;- brightness temperature (center wavelength > 2.5 microns)

if units eq 0 then begin
  if center( band - 1 ) lt 2.5 then begin
    image = temporary( image ) * ( !pi / ( irradiance( band - 1 ) * cos( !dtor * solzen ) ) )
  endif else begin
    image = bright_m( center( band - 1 ), temporary( image ) )
    if n_elements( tc_slope ) eq n_elements( center ) then $
	image = ( temporary( image ) - tc_intercept( band - 1 ) ) / tc_slope( band - 1 )
  endelse
endif

;- If radiance units are required (units = 1),
;- no action is required since image data is stored in radiance units

;- If count units are required (units = 2),
;- convert to counts using the calibration slope and intercept

if units eq 2 then begin
  calslope = rebin( temporary( calslope ), 716, scan_count )
  calintercept = rebin( temporary( calintercept ), 716, scan_count )
  image = ( temporary( image ) - temporary( calintercept ) ) / temporary( calslope )
endif

;- Close file

hdf_sd_end, sd_id

END
