PRO MASK_READ, FILE, DATE, TIME, NSCANS, MASK, SCENE = SCENE

;+
; Read a MAS cloud mask file,
; and make sure it corresponds to a given date, time, and number of scans
;
; Input:
; FILE    name of cloud mask file
; DATE    YYMMDD at start of corresponding HDF file
; TIME    HHMMSS UTC at start of corresponding HDF file
; NSCANS  number of scans in corresponding HDF file
;
; Output:
; MASK    array of mask values
;-

rcs_id = "$Id: mask_read.pro,v 1.3 2000/05/15 17:59:00 gumley Exp $"

;- Check keywords
if (n_elements(scene) eq 0) then scene = [0, 715]

;- Set return value
mask = -1

;- Read MAS cloud mask header
openr, lun, file, /get_lun
header = lonarr(4)
readu, lun, header
free_lun, lun

;- Check if byte swapping is required
if (header[1] lt 0) or (header[1] gt 235959) then begin
  header = swap_endian(header)
  if (header[1] lt 0) or (header[1] gt 235959) then begin
    result = dialog_message('Not a valid MAS cloud mask file')
    return
  endif
endif

;- Extract date, time, number of scans
file_date   = header[0]       ; date format is YYDDD
file_time   = header[1]       ; time format is HHMMSS (first scan in parent file)
file_nscans = header[3] - 1   ; number of scans in parent file minus two

;- Compare to input date, time, nscans
day = date mod 100L
month = ( date / 100L ) mod 100L
year = date / 10000L
doy = julday( year, month, day )
input_date = ( year mod 100L ) * 1000L + doy

if file_date ne input_date then begin
  string = string( file_date, input_date, format = $
    '( "Sorry: Cloud mask date (", i5, ") does not match MAS date (", i5, ")" )' )
  result = dialog_message( string )
  return
endif

if file_time ne time then begin
  string = string( file_time, time, format = $
    '( "Sorry: Cloud mask start time (", i6, ") does not match MAS start time (", i6, ")" )' )
  result = dialog_message( string )
endif

if ( file_nscans + 2 ) ne nscans then begin
  string = string( file_nscans + 2, nscans, format = $
    '( "Sorry: Cloud mask number of scans (", i5, ") does not match MAS number of scans (", i5, ")" )' )
  result = dialog_message( string )
  return
endif

;- Read cloud mask data
openr, lun, file, /get_lun
point_lun, lun, (scene[0] + 1) * 716L * 6L
data = bytarr(6, 716, 714)
readu, lun, data
free_lun, lun

;- Rotate into correct orientation in output array
mask = replicate(255B, 6, 716, 716)
for i = 0, 5 do begin
  mask( i, *, 1 : 714 ) = $
    rotate(rotate(reform(data[i, *, *], 716, 714), 7), 2)
endfor

end
