PRO READ_LOCATION, FILE, NLOC, LAT, LON, LABEL

;+
; Purpose:
;    Read a SHARP location file.
;
; Usage:
;    READ_LOCATION, FILE, NLOC, LAT, LON, LABEL
;
; Input:
;    FILE    Name of location file to read (see notes below for format)
;
; Output:
;    NLOC    Number of valid locations read
;    LAT     Array of latitude values (deg, -90S to +90N)
;    LON     Array of longitude values (deg, -180W to 180E)
;    LABEL   Array of location labels
;
; Revised:
;    Liam.Gumley@ssec.wisc.edu
;    $Id: read_location.pro,v 1.1 1999/11/16 17:23:22 gumley Exp $
;
; Notes:
;    The location file is an ASCII file containing lat, lon, and label
;    information for multiple locations.
;    (1) Any lines with the # character in the first column are comments
;    (2) Locations must specify a lat, lon, and label separated by commas
;    (3) Lines which contain other than 2 commas are ignored
;    (4) Lines which contain bad or missing values are ignored
;    (5) Latitude values outside the range -90 to 90 are ignored
;    (6) Longitude values outside the range -180 to 180 are ignored
;
;    The following example shows a file with 4 valid lines:
;# Test location file for SHARP
;34.887, -117.754, Dumbell
;34.916, -117.987, Main Ramp
;33.923 -117.232, Bad value
;35.272, -117.961, Center of Track
;34.558, -117.723, Lone Hill
;-

rcs_id = '$Id: read_location.pro,v 1.1 1999/11/16 17:23:22 gumley Exp $

;- Set default return value

nloc = 0L

;- Check arguments

if n_elements(file) eq 0 then begin
  message, 'FILE is undefined', /continue
  return
endif
if filecheck(file) eq 0 then begin
  message, 'FILE cannot be read', /continue
  return
endif

;- Open the file

openr, lun, file, /get_lun

;- Read each line in the file

lat = fltarr(1000)
lon = fltarr(1000)
label = strarr(1000)
while eof(lun) eq 0 do begin

  ;- Read this record

  record = ''
  readf, lun, record, format='(a)'
  if strmid(record, 0, 1) eq '#' then goto, continue

  ;- Extract the comma-separated items

  parts = str_sep(record, ',', /trim)

  ;- Extract the values

  valid = 0
  on_ioerror, bad_value
  if n_elements(parts) eq 3 then begin
    lat_value = float(parts[0])
    lon_value = float(parts[1])
    label_value = parts[2]
    valid = 1
    bad_value:
  endif
  on_ioerror, null

  ;- Store the values

  if valid eq 1 then begin
    if (lat_value ge  -90.0) and (lat_value le  90.0) and $
       (lon_value ge -180.0) and (lon_value le 180.0) then begin
      lat[nloc] = lat_value
      lon[nloc] = lon_value
      label[nloc] = label_value
      nloc = nloc + 1
    endif
  endif

  continue:

endwhile

;- Trim the result arrays

lat = lat[0 : nloc - 1]
lon = lon[0 : nloc - 1]
label = label[0 : nloc - 1]

;- Close the file

free_lun, lun

END
