PRO READ_MASTBL50, DATE, IFLAG, RC, HWN

rcs_id = "$Id: read_mastbl50.pro,v 1.2 1998/07/24 18:42:58 gumley Exp $"

;+
; PURPOSE:
;     Read a record from Chris Moeller's MCIDAS MAS 50 channel calibration table.
;
; INPUT:
;     DATE         Date in YYYYMMDD format
;
; OUTPUT:
;     IFLAG(50)    Array of count offset values for bands 1-50
;                  (valid for bands 1-25 only)
;     RC(50)       Array of radiance per count values for bands 1-50
;                  (valid for bands 1-25 only)
;     HWN(3,50)    Array of band centers and temperature corrections for bands 1-50
;                  (centers are valid for all bands)
;                  (temperature corrections are valid for bands 26-50)
;
; NOTES:
;
;     The calibration to radiance for MAS bands 1-25 is of the form
;     R = ( COUNT - IFLAG(BAND-1) ) * RC(BAND-1)
;     where
;     R is radiance in milliwatts per square centimeter per steradian per micron,
;     COUNT is a raw 15-bit count value (as stored in the UW Intermediate format),
;     BAND is the MAS band number from 1-25.
;
;     The format of array HWN is
;     Band  1: CENTER (inverse cm), OFFSET, SLOPE
;     Band  2: CENTER (inverse cm), OFFSET, SLOPE
;     ...
;     Band 50: CENTER (inverse cm), OFFSET, SLOPE
;
;     The conversion from brightness temperature to Planck radiance for
;     MAS bands 26-50 is of the form
;     R = B( CENTER(BAND-1), T*SLOPE(BAND-1) + OFFSET(BAND-1) )
;     where
;     R is the Planck radiance (units are determined by the Planck function),
;     B is the Planck function.
;
;-

;- convert date to YYDDD format

day = date mod 100L
month = ( date / 100L ) mod 100L
year = date / 10000L
newdate = ( year mod 100L ) * 1000L + julday( year, month, day )
diff = 10000L

;- open input file

file = 'mastbl50.xdr'
openr, lun_in, file, /get_lun, /xdr

;- read all entries and find closest date

flt = 0L
day = 0L
data = { iflag:lindgen(50), rc:fltarr(50), scale:fltarr(50), hwn:fltarr(3,50) }

;print, "Searching mastbl50.xdr for a calibration entry with date (YYDDD) = ", $
;  strcompress( newdate, /remove_all )
;print, "   YYFFF   YYDDD"
;print, "   -----   -----"

while not eof( lun_in ) do begin

  ;- read the data, and save data if we get a good date match
  
  readu, lun_in, flt, day
  readu, lun_in, data
  
  if not( ( flt mod 1000 ) gt 200 or $
          ( day mod 1000 ) gt 366 or $
          flt/1000 ne day/1000 ) then begin
    if abs( day - newdate ) lt diff then begin
	diff = abs( day - newdate )
	iflag = data.iflag
	rc = data.rc
	hwn = data.hwn
;	print, flt, day, format = '( 2( 3x, i5 ) )'
    endif    
  endif
    
endwhile

;- close files

free_lun, lun_in

END
