PRO REGION_CALIBRATE, IMAGE, BAND, SOLZEN, IRRADIANCE, CENTER, TC_SLOPE, TC_INTERCEPT

;- Return to caller if error occurs

on_error, 2

;- Check arguments

if n_elements( image )      eq 0 then message, 'Argument IMAGE is missing'
if n_elements( band )       eq 0 then message, 'Argument BAND is missing'
if n_elements( solzen )     eq 0 then message, 'Argument SOLZEN is missing'
if n_elements( irradiance ) eq 0 then message, 'Argument IRRADIANCE is missing'
if n_elements( center )     eq 0 then message, 'Argument CENTER is missing'

;- Convert to reflectance or brightness temperature

if center( band - 1 ) lt 2.5 then begin

  ;- Convert visible/near-IR bands from radiance to reflectance

  image = temporary( image ) * $
    ( !pi / ( irradiance( band - 1 ) * cos( !dtor * solzen ) ) )

endif else begin

  ;- Convert IR bands from radiance to brightness temperature

  image = bright_m( center( band - 1 ), temporary( image ) )
  if n_elements( tc_slope ) eq n_elements( center ) then $
    image = ( temporary( image ) - tc_intercept( band - 1 ) ) / tc_slope( band - 1 )

endelse

END

;##############################################################################

PRO REGION_READ_IMAGE, FILE, SCENE, NBANDS, $
  IMAGE

;- Return to caller if error occurs

on_error, 2

;- Check arguments

if n_elements( file )       eq 0 then message, 'Argument FILE is missing'
if n_elements( scene ) eq 0 then message, 'Argument SCENE is missing'
if n_elements( nbands )     eq 0 then message, 'Argument NBANDS is missing'

;- Compute record range

record = scene( 0 )
nrecords = scene( 1 ) - scene( 0 ) + 1

;- Open file

sd_id = hdf_sd_start( file )

;- Get image data for all bands

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
start = [ 0, 0, record ]
count = [ 716, nbands, nrecords ]
hdf_sd_getdata, sds_id, image, start = start, count = count
hdf_sd_attrinfo, sds_id, hdf_sd_attrfind( sds_id, 'scale_factor' ), data = scale
hdf_sd_endaccess, sds_id

;- Close file

hdf_sd_end, sd_id

;- Apply scale factors and rotate image array into correct orientation

image = float( temporary( image ) )
for band = 1, nbands do begin
  data = reform( image( *, band - 1, * ) )
  data = temporary( data ) * scale( band - 1 )
  data = rotate( rotate( temporary( data ), 7 ), 2 )
  image( *, band - 1, * ) = temporary( data )
endfor

;- Make sure there are no negative values in the image

image = temporary( image ) > 1.0e-6

END

;##############################################################################

PRO REGION_READ_CALIBRATION, FILE, SCENE, $
  SOLZEN, IRRADIANCE, CENTER, TC_SLOPE, TC_INTERCEPT

;- Return to caller if error occurs

on_error, 2

;- Check arguments

if n_elements( file )  eq 0 then message, 'Argument FILE is missing'
if n_elements( scene ) eq 0 then message, 'Argument SCENE is missing'

;- Compute record range

record = scene( 0 )
nrecords = scene( 1 ) - scene( 0 ) + 1

;- Open file

sd_id = hdf_sd_start( file )

;- Get solar zenith

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarZenithAngle' ) )
hdf_sd_getinfo, sds_id, dims = dims
hdf_sd_getdata, sds_id, solzen, start = [ 0, record ], count = [ dims( 0 ), nrecords ]
hdf_sd_endaccess, sds_id
if dims( 0 ) lt 716 then solzen = congrid( temporary( solzen ), 716, nrecords, /interp )
solzen = rotate( rotate( temporary( solzen ), 7 ), 2 )

;- Get solar spectral irradiances

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'SolarSpectralIrradiance' ) )
hdf_sd_getdata, sds_id, irradiance
hdf_sd_endaccess, sds_id

;- Get center wavelengths

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'Central100%ResponseWavelength' ) )
hdf_sd_getdata, sds_id, center
hdf_sd_endaccess, sds_id
index = hdf_sd_nametoindex( sd_id, 'EffectiveCentralWavelength_IR_Bands' )
if index gt 0 then begin
  sds_id = hdf_sd_select( sd_id, index )
  hdf_sd_getdata, sds_id, tc_center
  hdf_sd_endaccess, sds_id
  center( 25 : 49 ) = temporary( tc_center( 25 : 49 ) )
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionSlope' ) )
  hdf_sd_getdata, sds_id, tc_slope
  hdf_sd_endaccess, sds_id
  sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'TemperatureCorrectionIntercept' ) )
  hdf_sd_getdata, sds_id, tc_intercept
  hdf_sd_endaccess, sds_id
endif

;- Close file

hdf_sd_end, sd_id

END

;##############################################################################

PRO REGION_READ, FILE, SCENE, PIXEL, NPIXELS, IMAGE, UNITS = UNITS

rcs_id = "$Id: region_read.pro,v 1.2 1998/10/27 20:37:30 gumley Exp $"

;- Return to caller if error occurs

on_error, 2

;- Check arguments

if n_elements( file )     eq 0 then message, 'Argument FILE is missing'
if n_elements( scene ) eq 0 then message, 'Argument SCENE is missing'
if n_elements( pixel )    eq 0 then message, 'Argument PIXEL is missing'
if n_elements( npixels )  eq 0 then message, 'Argument NPIXELS is missing'

;- Check keywords

if n_elements( units ) ne 1 then units = 0

;- Compute record range

record = scene( 0 )
nrecords = scene( 1 ) - scene( 0 ) + 1

;- Get calibration information for this region

region_read_calibration, file, scene, $
  solzen, irradiance, center, tc_slope, tc_intercept
nbands = n_elements( irradiance )

;- Subset by solar zenith array by pixel

solzen = temporary( solzen( pixel : pixel + npixels - 1, * ) )

;- Get image data for this region in chunks of 10 scanlines

image = fltarr( 716, nbands, scene( 1 ) - scene( 0 ) + 1, /nozero )
scan_start = scene( 0 )
scan_end   = scene( 1 )
scan_step  = 10
scan0 = scan_start
scan1 = ( scan_start + scan_step - 1 ) < scan_end

while scan0 lt scan_end do begin
  region_read_image, file, [ scan0, scan1 ], nbands, data
  image( *, *, scan0 - scan_start : scan1 - scan_start  ) = temporary( data )
  scan0 = scan0 + scan_step
  scan1 = ( scan1 + scan_step ) < scan_end
endwhile

;- Subset the image array by pixel

image = temporary( image( pixel : pixel + npixels - 1, *, * ) )

;- Calibrate image data

if units eq 0 then begin
  for band = 1, nbands do begin
    data = reform( image( *, band - 1, * ) )
    region_calibrate, data, band, solzen, irradiance, center, tc_slope, tc_intercept
    image( *, band - 1, * ) = temporary( data )
  endfor
endif

END
