FUNCTION SAFE_TVRD, X0, Y0, NX, NY, CHANNEL, _EXTRA = _EXTRA

rcs_id = "$Id: safe_tvrd.pro,v 1.2 1998/09/29 19:37:22 gumley Exp $"

;+
;PURPOSE:
;    The SAFE_TVRD function returns the contents of the specified
;    rectangular portion of the current graphics window or device. (X0, Y0) is
;    the coordinate of the lower left corner of the area to be read and Nx, Ny
;    is the size of the rectangle in columns and rows. The result is a byte
;    array of dimensions Nx by Ny. All parameters are optional. If no arguments
;    are supplied, the entire display device area is read.
;
;CALLING SEQUENCE:
;    Result = SAFE_TVRD([X0 [, Y0 [, Nx [, Ny [, Channel]]]]])
;
;ARGUMENTS:
;    X0         The starting column of data to read. The default is 0.
;
;    Y0         The starting row of data to read. The default is 0.
;
;    NX         The number of columns to read. The default is the width of the display
;               device or window less X0.
;
;    NY         The number of rows to read. The default is the height of the display
;               device or window less Y0.
;
;    CHANNEL    The memory channel to be read. If not specified, this argument is
;               assumed to be zero. This parameter is ignored on display systems
;               that have only one memory channel.
;
;KEYWORDS:
;    CHANNEL    The memory channel to be read. The CHANNEL keyword is identical to
;               the optional CHANNEL argument.
;               Note: if the display is a 24-bit display, and both the CHANNEL and
;               TRUE parameters are absent, the maximum RGB value in each pixel is
;               returned.
;
;    ORDER      Set this keyword to override the current setting of the !ORDER system
;               variable for the current image only. If set, it causes the image to be read
;               from the top down instead of the normal bottom up.
;   
;    TRUE       If this keyword is present, it indicates that a true-color image is to
;               be read, if the display is capable. The value assigned to TRUE specifies
;               the index of the dimension over which color is interleaved. The result is
;               an (3, nx, ny) pixel-interleaved array if TRUE is 1; or an (nx, 3, ny)
;               line-interleaved array if TRUE is 2; or an (nx, ny, 3) image-interleaved
;               array if TRUE is 3.
;
;    WORDS      Set this keyword to indicate that words are to be transferred from
;               the device. This keyword is valid only when using devices that can transfer
;               16-bit pixels. The normal transfer uses 8-bit pixels. If this keyword is
;               set, the function result is an integer array.
;
;AUTHOR:
;    Liam.Gumley@ssec.wisc.edu
;-

;- Check that current display window is valid

if !d.window eq -1 then message, 'Invalid window ID'

;- Check arguments

if n_elements( x0 ) eq 0 then x0 = 0
if n_elements( y0 ) eq 0 then y0 = 0
if n_elements( nx ) eq 0 then nx = !d.x_size - x0
if n_elements( ny ) eq 0 then ny = !d.y_size - y0
if n_elements( channel ) eq 0 then channel = 0

;- Save current window settings

current_window = !d.window
current_x_size = !d.x_size
current_y_size = !d.y_size

;- Create pixmap window

window, /free, /pixmap, xsize = current_x_size, ysize = current_y_size

;- Copy contents of display window into pixmap window

device, copy = [ x0, y0, nx, ny, x0, y0, current_window ]

;- Use TVRD to get contents of pixmap window

result = tvrd( x0, y0, nx, ny, channel, _extra = _extra )

;- Delete pixmap window and make display window current

wdelete, !d.window
wset, current_window

;- Return result to calling program

return, temporary( result )

END
